import numpy as np
import matplotlib.pyplot as plt


def afficher_grille_simple(grille):
    """
    Cette fonction permet d'afficher une grille sans faire figurer les couronnes et les croix

    Paramètres:
    grille (list(list(str))) : grille avec les couleurs de chaque case

    """

    n_rows = len(grille)
    n_cols = len(grille[0])
    
    # Création de la figure et des axes
    fig, ax = plt.subplots(figsize=(n_cols, n_rows))
    ax.set_xticks(np.arange(n_cols + 1), minor=True)
    ax.set_yticks(np.arange(n_rows + 1), minor=True)
    ax.grid(which="minor", color="black", linestyle='-', linewidth=1)
    ax.tick_params(which="both", bottom=False, left=False, labelbottom=False, labelleft=False)

    # Dessin des cellules colorées
    for i in range(n_rows):
        for j in range(n_cols):
            ax.add_patch(plt.Rectangle((j, n_rows - 1 - i), 1, 1, color=grille[i][j], ec='black', lw=2))
    
    # Ajout des bordures épaisses entre couleurs différentes
    for i in range(n_rows):
        for j in range(n_cols):
            if j < n_cols - 1 and grille[i][j] != grille[i][j + 1]:  # Bord vertical droit
                ax.plot([j + 1, j + 1], [n_rows - i, n_rows - 1 - i], color='black', linewidth=4)
            if i < n_rows - 1 and grille[i][j] != grille[i + 1][j]:  # Bord horizontal bas
                ax.plot([j, j + 1], [n_rows - 1 - i, n_rows - 1 - i], color='black', linewidth=4)
    
    # Bordures extérieures plus épaisses
    ax.plot([0, n_cols], [n_rows, n_rows], color='black', linewidth=6)  # Bord haut
    ax.plot([0, n_cols], [0, 0], color='black', linewidth=6)  # Bord bas
    ax.plot([0, 0], [0, n_rows], color='black', linewidth=6)  # Bord gauche
    ax.plot([n_cols, n_cols], [0, n_rows], color='black', linewidth=6)  # Bord droit

    plt.xlim(0, n_cols)
    plt.ylim(0, n_rows)
    plt.gca().set_aspect('equal')  # Assurer un affichage carré des cellules
    plt.show()

def afficher_grille(grille, variables, result):
    """
    Cette fonction permet d'afficher une grille en faisant apparaître les croix et couronnes.

    Paramètres:
    grille (list(list(str))) : grille avec les couleurs de chaque case
    variables: (list(list(str))) : grille avec les variables associées à chaque case
    result (dict(int : int)) : dictionnaire renseignant la valeur de vérité des variables (1: couronne, 0: pas de couronne)
    
    """

    n_rows = len(grille)
    n_cols = len(grille[0])
    
    # Création de la figure et des axes
    fig, ax = plt.subplots(figsize=(n_cols, n_rows))
    ax.set_xticks(np.arange(n_cols + 1), minor=True)
    ax.set_yticks(np.arange(n_rows + 1), minor=True)
    ax.grid(which="minor", color="black", linestyle='-', linewidth=1)
    ax.tick_params(which="both", bottom=False, left=False, labelbottom=False, labelleft=False)

    # Dessin des cellules colorées
    for i in range(n_rows):
        for j in range(n_cols):
            ax.add_patch(plt.Rectangle((j, n_rows - 1 - i), 1, 1, color=grille[i][j], ec='black', lw=2))
            
            # Dessiner la couronne ou la croix en fonction de matrix[i][j]
            if result[variables[i][j]] == 1:
                # Couronne (Cercle non rempli)
                circle = plt.Circle((j + 0.5, n_rows - 1 - i + 0.5), 0.3, color='black', fill=False, linewidth=2)
                ax.add_patch(circle)
            elif result[variables[i][j]] == 0:
                # Petite croix (deux lignes diagonales)
                ax.plot([j + 0.3, j + 0.7], [n_rows - 1 - i + 0.3, n_rows - 1 - i + 0.7], color='black', linewidth=1)
                ax.plot([j + 0.3, j + 0.7], [n_rows - 1 - i + 0.7, n_rows - 1 - i + 0.3], color='black', linewidth=1)
    
    # Ajout des bordures épaisses entre couleurs différentes
    for i in range(n_rows):
        for j in range(n_cols):
            if j < n_cols - 1 and grille[i][j] != grille[i][j + 1]:  # Bord vertical droit
                ax.plot([j + 1, j + 1], [n_rows - i, n_rows - 1 - i], color='black', linewidth=4)
            if i < n_rows - 1 and grille[i][j] != grille[i + 1][j]:  # Bord horizontal bas
                ax.plot([j, j + 1], [n_rows - 1 - i, n_rows - 1 - i], color='black', linewidth=4)
    
    # Bordures extérieures plus épaisses
    ax.plot([0, n_cols], [n_rows, n_rows], color='black', linewidth=6)  # Bord haut
    ax.plot([0, n_cols], [0, 0], color='black', linewidth=6)  # Bord bas
    ax.plot([0, 0], [0, n_rows], color='black', linewidth=6)  # Bord gauche
    ax.plot([n_cols, n_cols], [0, n_rows], color='black', linewidth=6)  # Bord droit

    plt.xlim(0, n_cols)
    plt.ylim(0, n_rows)
    plt.gca().set_aspect('equal')  # Assurer un affichage carré des cellules
    plt.show()