from .fonctions_base import row_columns_conditions, neighbors_conditions, create_variable, process_sat_output
from .generation_and_solver import getDirectColoredNeighbors
from random import randint, choice
from .affichage import afficher_grille
import subprocess


def Génération_grille_illutrée(n):
    """
    Cette fonction est similaire a génération grille. Mais elle affiche également la grille à chaque étape
    de la génération.

    Paramètres:
    n (int) : dimensions de la grille

    Retourne:
    list(list(int)) : un grille de taille nxn dont chaque chiffre correspond à une couleur
    
    """
    grille_initiale = [[0 for i in range(n)] for i in range(n)]
    variables = create_variable(grille_initiale)
    
    # Donner les conditions sans celle des couleurs
    conditions = ""
    conditions += row_columns_conditions(variables)
    conditions += neighbors_conditions(variables)

    # Ecrire le fichier cnf
    random_var = f"{choice(choice(variables))} 0"
    nb_clauses = conditions.count('0') + 1
    en_tete = f'p cnf {n**2} {nb_clauses}\n'
    with open('creation_grille.cnf','w') as file:
        file.write(en_tete + random_var + conditions)
    
    # Lancer gophersat
    path_to_gophersat = './gophersat_win64'
    command = [path_to_gophersat,'creation_grille.cnf']
    result = subprocess.run(command, capture_output=True, text=True)
    result = process_sat_output(result.stdout)
    
    # Affecter une couleur à chaque couronne
    cpt = 1
    val_couleur = {}
    repartitions_zone = {i : [] for i in range(n+1)}

    decode_color = {
    0: 'white',
    1: 'red',
    2: 'blue',
    3: 'green',
    4: 'yellow',
    5: 'cyan',
    6: 'magenta',
    7: 'lightgreen',
    8: 'lightblue',
    9: 'gray',
    10: 'orange',
    11: 'purple',
    12: 'brown',
    13: 'pink',
    14: 'lime',
    15: 'olive',
    16: 'navy',
    17: 'teal',
    18: 'maroon',
    19: 'gold',
    20: 'violet'
    }

    for i in range(n):
        for j in range(n):
            if result[variables[i][j]] == 1:
                grille_initiale[i][j] += cpt
                cpt +=1    
            val_couleur[variables[i][j]] = grille_initiale[i][j]
            repartitions_zone[grille_initiale[i][j]].append(variables[i][j])


    grille_courante = [[decode_color[cell] for cell in row] for row in grille_initiale]
    afficher_grille(grille_courante,variables,result)

    while len(repartitions_zone[0]) != 0:
        nb_non_coloré = len(repartitions_zone[0])
        if nb_non_coloré>1:
            candidat = repartitions_zone[0][randint(0,nb_non_coloré-1)]
        else:
            candidat = repartitions_zone[0][0]
        voisins = getDirectColoredNeighbors(candidat,n,val_couleur)
        nb_voisins = len(voisins)
        if nb_voisins > 0:
            # On choisit au hasard un voisin
            if nb_voisins > 1:
                voisin_choisi = voisins[randint(0,nb_voisins-1)]
            else:
                voisin_choisi = voisins[0]
            nouvelle_couleur = val_couleur[voisin_choisi]

            # On affecte la couleur au centre et on actualise la répartition des couleurs
            val_couleur[candidat] = nouvelle_couleur
            repartitions_zone[nouvelle_couleur].append(candidat)
            repartitions_zone[0].remove(candidat)

            # Afficher la grille courante
            grille_courante = [[decode_color[val_couleur[var]] for var in row] for row in variables]
            afficher_grille(grille_courante,variables,result)

    return [[val_couleur[var] for var in row] for row in variables]